@@ -21,26 +21,36 @@ def init_registry():
         _initialized = True
 
 
-def load_modules(modules=None):
+def _has_django():
     try:
         import django  # noqa
+        return True
     except ImportError:
         # Not a django project
+        pass
+    return False
+
+
+def _autodiscover_modules(module_name):  # pragma: no cover
+    "Django 1.6 compat to provide `autodiscover_modules`"
+    from django.conf import settings
+    from django.utils.importlib import import_module
+
+    for app in settings.INSTALLED_APPS:
+        # Attempt to import the app's `module_name`.
+        try:
+            import_module('{app}.{module}'.format(app=app, module=module_name))
+        except Exception:
+            pass
+
+
+def load_modules(modules=None):
+    if not _has_django():
         return
     try:  # pragma: no cover
         from django.utils.module_loading import autodiscover_modules
     except ImportError:  # pragma: no cover
-        # Django 1.6 compat to provide `autodiscover_modules`
-        def autodiscover_modules(module_name):
-            from django.conf import settings
-            from django.utils.importlib import import_module
+        autodiscover_modules = _autodiscover_modules
 
-            for app in settings.INSTALLED_APPS:
-                # Attempt to import the app's `module_name`.
-                try:
-                    import_module('{app}.{module}'.format(app=app, module=module_name))
-                except Exception:
-                    pass
-
-        for module in modules:
-            autodiscover_modules(module)
+    for module in modules:
+        autodiscover_modules(module)
