@@ -1,12 +1,17 @@
 # coding: utf-8
 
-from collections import OrderedDict
 from . import registry
-try:
-    from django.utils.translation import ugettext as _
-except Exception:
-    def _(text):
-        return text
+from .exceptions import (
+    StateMachineError,
+    InvalidDefinition,
+    InvalidStateValue,
+    InvalidDestinationState,
+    InvalidTransitionIdentifier,
+    TransitionNotAllowed,
+    MultipleStatesFound,
+    MultipleTransitionCallbacksFound,
+)
+from .utils import ugettext as _
 
 
 class CallableInstance(object):
@@ -74,56 +79,47 @@ class Transition(object):
         self.managed = managed
         self.identifier = identifier
 
-    def __get__(self, instance, owner):
-        def callable(*args, **kwargs):
-            return self._run(instance, *args, **kwargs)
+    def __get__(self, machine, owner):
+        def transition_callback(*args, **kwargs):
+            return self._run(machine, *args, **kwargs)
 
-        return CallableInstance(self, func=callable)
+        return CallableInstance(self, func=transition_callback)
 
     def __set__(self, instance, value):
         "does nothing (not allow overriding)"
 
     def __call__(self, f):
         if not callable(f):
-            raise ValueError('Transitions can only be called as method decorators.')
+            raise StateMachineError('Transitions can only be called as method decorators.')
         self.on_execute = f
         return self
 
-    def _can_run(self, instance):
-        return instance.current_state == self.source
+    def _can_run(self, machine):
+        if machine.current_state == self.source:
+            return self
 
-    def _verify_can_run(self, instance):
-        if not self._can_run(instance):
-            raise LookupError(
-                _("Can't {} when in {}.").format(
-                    self.identifier,
-                    instance.current_state.name
-                )
-            )
+    def _verify_can_run(self, machine):
+        transition = self._can_run(machine)
+        if not transition:
+            raise TransitionNotAllowed(self, machine.current_state)
+        return transition
 
-    def _run(self, instance, *args, **kwargs):
-        self._verify_can_run(instance)
+    def _run(self, machine, *args, **kwargs):
+        self._verify_can_run(machine)
         self._validate(*args, **kwargs)
-        return instance._activate(self, *args, **kwargs)
+        return machine._activate(self, *args, **kwargs)
 
     def _validate(self, *args, **kwargs):
         for validator in self.validators:
             validator(*args, **kwargs)
 
-    def _get_destination(self, result):
+    def _get_destination_from_result(self, result):
         """
-        A transition can point to one or more destination states.
-        If there is more than 1 state, the transition **must** specify a `on_execution` callback
-        that should return the desired state.
+        Try to extract a destination state from ``result`` if it exists.
         """
-        msg_multiple = _('Multiple destinations, you must return the desired state as: '
-                         '`return <State>` or `return <result>, <State>`.')
-
-        if len(self.destinations) == 1:
-            return result, self.destinations[0]
-
+        destination = None
         if result is None:
-            raise ValueError(msg_multiple)
+            return result, destination
         elif isinstance(result, State):
             destination = result
             result = None
@@ -131,19 +127,34 @@ class Transition(object):
             try:
                 num = len(result)
             except TypeError:
-                raise ValueError(msg_multiple)
+                return result, destination
 
-            if num != 2:
-                raise ValueError(msg_multiple)
+            if num < 2:
+                return result, destination
 
-            result, destination = result
+            if isinstance(result[-1], State):
+                result, destination = result[:-1], result[-1]
+                if len(result) == 1:
+                    result = result[0]
 
-        if not isinstance(destination, State):
-            raise ValueError(msg_multiple)
+        return result, destination
+
+    def _get_destination(self, result):
+        """
+        A transition can point to one or more destination states.
+        If there is more than 1 state, the transition **must** specify a `on_execution` callback
+        that should return the desired state.
+        """
+        result, destination = self._get_destination_from_result(result)
+
+        if destination is None:
+            if len(self.destinations) == 1:
+                destination = self.destinations[0]
+            else:
+                raise MultipleStatesFound(self)
 
         if destination not in self.destinations:
-            raise ValueError(
-                _('{} is not a possible destination state.').format(destination.name))
+            raise InvalidDestinationState(self, destination)
 
         return result, destination
 
@@ -169,14 +180,13 @@ class CombinedTransition(Transition):
         self._left.__contribute_to_class__(managed, identifier)
         self._right.__contribute_to_class__(managed, identifier)
 
-    def _can_run(self, instance):
-        return instance.current_state in [self._left.source, self._right.source]
+    def _can_run(self, machine):
+        return self._left._can_run(machine) or self._right._can_run(machine)
 
-    def _run(self, instance, *args, **kwargs):
-        self._verify_can_run(instance)
+    def _run(self, machine, *args, **kwargs):
+        transition = self._verify_can_run(machine)
         self._validate(*args, **kwargs)
-        transition = self._left if instance.current_state == self._left.source else self._right
-        return transition._run(instance, *args, **kwargs)
+        return transition._run(machine, *args, **kwargs)
 
 
 class State(object):
@@ -192,10 +202,18 @@ class State(object):
         return "{}({!r}, identifier={!r}, value={!r}, initial={!r})".format(
             type(self).__name__, self.name, self.identifier, self.value, self.initial)
 
-    def to(self, *states):
-        transition = Transition(self, *states)
-        self.transitions.append(transition)
-        return transition
+    @property
+    def to(self):
+        def to_method(*states):
+            transition = Transition(self, *states)
+            self.transitions.append(transition)
+            return transition
+
+        def to_itself():
+            return to_method(self)
+
+        to_method.itself = to_itself
+        return to_method
 
     def __contribute_to_class__(self, managed, identifier):
         self.managed = managed
@@ -254,21 +272,22 @@ class BaseStateMachine(object):
         self.check()
 
     def __repr__(self):
-        return "{}(model={!r}, state_field={!r})".format(
+        return "{}(model={!r}, state_field={!r}, current_state={!r})".format(
             type(self).__name__, self.model, self.state_field,
+            self.current_state.identifier,
         )
 
     def check(self):
         if not self.states:
-            raise ValueError(_('There are no states.'))
+            raise InvalidDefinition(_('There are no states.'))
 
         if not self.transitions:
-            raise ValueError(_('There are no transitions.'))
+            raise InvalidDefinition(_('There are no transitions.'))
 
         initials = [s for s in self.states if s.initial]
         if len(initials) != 1:
-            raise ValueError(_('There should be one and only one initial state. '
-                               'Your currently have these: {!r}'.format(initials)))
+            raise InvalidDefinition(_('There should be one and only one initial state. '
+                                      'Your currently have these: {!r}'.format(initials)))
         self.initial_state = initials[0]
 
         if self.current_state_value is None:
@@ -284,7 +303,7 @@ class BaseStateMachine(object):
     @current_state_value.setter
     def current_state_value(self, value):
         if value not in self.states_map:
-            raise ValueError(_("{!r} is not a valid state value.").format(value))
+            raise InvalidStateValue(value)
         setattr(self.model, self.state_field, value)
 
     @property
@@ -304,19 +323,45 @@ class BaseStateMachine(object):
         self.current_state_value = value.value
 
     def _activate(self, transition, *args, **kwargs):
-        on_event = getattr(self, 'on_{}'.format(transition.identifier), transition.on_execute)
-        result = on_event(self, *args, **kwargs) if callable(on_event) else None
+        bounded_on_event = getattr(self, 'on_{}'.format(transition.identifier), None)
+        on_event = transition.on_execute
+
+        if bounded_on_event and on_event and bounded_on_event != on_event:
+            raise MultipleTransitionCallbacksFound(transition)
+
+        result = None
+        if callable(bounded_on_event):
+            result = bounded_on_event(*args, **kwargs)
+        elif callable(on_event):
+            result = on_event(self, *args, **kwargs)
 
         result, destination = transition._get_destination(result)
 
+        bounded_on_exit_state_event = getattr(self, 'on_exit_state', None)
+        if callable(bounded_on_exit_state_event):
+            bounded_on_exit_state_event(self.current_state)
+
+        bounded_on_enter_state_event = getattr(self, 'on_enter_state', None)
+        if callable(bounded_on_enter_state_event):
+            bounded_on_enter_state_event(destination)
+
+        bounded_on_exit_specific_state_event = getattr(
+            self, 'on_exit_{}'.format(self.current_state.identifier), None)
+        if callable(bounded_on_exit_specific_state_event):
+            bounded_on_exit_specific_state_event()
+
+        bounded_on_enter_specific_state_event = getattr(
+            self, 'on_enter_{}'.format(destination.identifier), None)
+        if callable(bounded_on_enter_specific_state_event):
+            bounded_on_enter_specific_state_event()
+
         self.current_state = destination
         return result
 
     def get_transition(self, transition_identifier):
         transition = getattr(self, transition_identifier, None)
         if not hasattr(transition, 'source') or not callable(transition):
-            raise ValueError(
-                '{!r} is not a valid transition identifier'.format(transition_identifier))
+            raise InvalidTransitionIdentifier(transition_identifier)
         return transition
 
     def run(self, transition_identifier, *args, **kwargs):
